@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 
-# Copyright (c) 2016-2019, Magnus Edenhill
+# Copyright (c) 2016-2022, Magnus Edenhill
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@
 #  * KerberosKdcApp (optional, sasl.mechanism=GSSAPI,
 #                    cross-realm if realm_cnt=2)
 #  * SchemaRegistryApp (optional, if with_sr=True)
+#  * OauthbearerOIDCApp (optional, if oidc=True)
 #
 # cluster.env (dict) will contain:
 #      TRIVUP_ROOT
@@ -54,12 +55,14 @@ from trivup.apps.KafkaBrokerApp import KafkaBrokerApp
 from trivup.apps.KerberosKdcApp import KerberosKdcApp
 from trivup.apps.SchemaRegistryApp import SchemaRegistryApp
 from trivup.apps.SslApp import SslApp
+from trivup.apps.OauthbearerOIDCApp import OauthbearerOIDCApp
 
 from copy import deepcopy
 
 import os
 import argparse
 import subprocess
+import copy
 
 
 class KafkaCluster(object):
@@ -83,6 +86,9 @@ class KafkaCluster(object):
         'with_sr': False,
         # Debug trivup
         'debug': False,
+        # Cleanup
+        'cleanup': True,
+        'oidc': False,
         # Additional broker server.properties configuration
         # 'broker_conf': ['connections.max.idle.ms=1234', ..]
     }
@@ -98,19 +104,29 @@ class KafkaCluster(object):
             self.conf.update(conf)
 
         self.version = self.conf.get('version')
+        self.version_num = [int(x) for x in self.version.split('.')][:3]
         self.kraft = self.conf.get('kraft')
 
         # Create trivup Cluster
         self.cluster = Cluster(
             self.__class__.__name__,
             os.environ.get('TRIVUP_ROOT', 'tmp-%s' % self.__class__.__name__),
-            debug=bool(self.conf.get('debug', False)))
+            debug=bool(self.conf.get('debug', False)),
+            cleanup=bool(self.conf.get('cleanup', True)))
 
         self._client_conf = dict()
         self.env = dict()
 
         self.sasl_mechanism = self.conf.get('sasl_mechanism')
 
+        # Add OIDC server app
+        if bool(self.conf.get('oidc', False)):
+            self.oidc = OauthbearerOIDCApp(self.cluster)
+            if not self.sasl_mechanism:
+                self.sasl_mechanism = 'OAUTHBEARER'
+            elif self.sasl_mechanism.upper() != 'OAUTHBEARER':
+                raise RuntimeError(f"OIDC requires sasl.mechanism OAUTHBEARER, not '{self.sasl_mechanism}'")
+
         # Generate SSL certs if enabled
         if bool(self.conf.get('with_ssl')):
             self.ssl = SslApp(self.cluster, self.conf)
@@ -153,7 +169,16 @@ class KafkaCluster(object):
         # Create brokers (don't start yet)
         self.brokers = dict()
         for n in range(0, broker_cnt):
-            broker = KafkaBrokerApp(self.cluster, self.broker_conf)
+            bconf = copy.deepcopy(self.broker_conf)
+            if self.version_num >= [2, 4, 0]:
+                # Configure rack & replica selector if broker supports
+                # fetch-from-follower
+                bconf.update(
+                    { 'conf': [
+                        'broker.rack=RACK${appid}',
+                        'replica.selector.class=org.apache.kafka.common.replica.RackAwareReplicaSelector']})  # noqa: E501
+
+            broker = KafkaBrokerApp(self.cluster, bconf)
             self.brokers[broker.appid] = broker
 
         # Generate bootstrap servers list
@@ -225,9 +250,18 @@ class KafkaCluster(object):
                     break
 
             elif self.sasl_mechanism == 'OAUTHBEARER':
-                self._client_conf['enable.sasl.oauthbearer.unsecure.jwt'] = True  # noqa: E501
-                self._client_conf['sasl.oauthbearer.config'] = \
-                    'scope=requiredScope principal=admin'
+                if self.oidc is not None:
+                    self._client_conf['sasl.oauthbearer.method'] = 'OIDC'
+                    self._client_conf['sasl.oauthbearer.token.endpoint.url'] = self.oidc.get('valid_url')  # noqa: E501
+                    self._client_conf['sasl.oauthbearer.client.id'] = '123'
+                    self._client_conf['sasl.oauthbearer.client.secret'] = 'abc'
+                    self._client_conf['sasl.oauthbearer.scope'] = 'test'
+                    self._client_conf['sasl.oauthbearer.extensions'] = \
+                        'ExtensionworkloadIdentity=develC348S,Extensioncluster=lkc123'
+                else:
+                    self._client_conf['enable.sasl.oauthbearer.unsecure.jwt'] = True  # noqa: E501
+                    self._client_conf['sasl.oauthbearer.config'] = \
+                        'scope=requiredScope principal=admin'
 
         # Client SSL configuration
         if self.ssl is not None:
@@ -435,6 +469,9 @@ if __name__ == '__main__':
     parser.add_argument('--cpversion', dest='cp_version', type=str,
                         default=KafkaCluster.default_conf['cp_version'],
                         help='Confluent Platform version (for Schema-Registry)')  # noqa: E501
+    parser.add_argument('--no-cleanup', dest='no_cleanup', action='store_true',
+                        default=False,
+                        help='Don\'t clean up logs, etc, after cluster is destroyed')
     parser.add_argument('--cmd', type=str, dest='cmd', default=None,
                         help='Command to execute instead of interactive shell')
     parser.add_argument('--kafka-src', dest='kafka_src', type=str,
@@ -442,6 +479,10 @@ if __name__ == '__main__':
                         help='Existing Kafka source directory to use ' +
                         'instead of downloaded release. ' +
                         'Requires --version trunk.')
+    parser.add_argument('--oidc', dest='oidc',
+                        action='store_true',
+                        default=KafkaCluster.default_conf['oidc'],
+                        help='Enable Oauthbearer OIDC JWT server')
 
     args = parser.parse_args()
 
@@ -453,7 +494,9 @@ if __name__ == '__main__':
             'with_ssl': args.ssl,
             'with_sr': args.sr,
             'broker_cnt': args.broker_cnt,
-            'kafka_path': args.kafka_src}
+            'kafka_path': args.kafka_src,
+            'cleanup': not args.no_cleanup,
+            'oidc': args.oidc}
 
     kc = KafkaCluster(**conf)
 
